import java.awt.event.*;
import javax.swing.*;
import java.awt.BorderLayout;
import java.io.IOException;

/**
 * Lab #6 
 * CS 2334, Section 0?? 
 * DATE GOES HERE
 * <P>
 * This class implements a GUI based program that allows an inventory of words
 * to the manipulated.
 * </P>
 * 
 * @author Group member names??
 * @version 1.0
 */
public class WordGUI implements ActionListener {
	/** Main frame for the program. */
	private JFrame frame;
	/** List to display the movies in. */
	private JList words;
	/** Button to add a word. */
	private JButton addButton;
	/** Button to remove a word. */
	private JButton removeButton;
	/** Data model for holding the words. */
	private WordListModel listModel;

	/**
	 * Initializes the data model for the program and initializes all GUI
	 * components associated with the program.
	 * <P>
	 * 
	 * @param appFrame
	 *            JFrame to use as the window for the program.
	 * @param model
	 *            Data model for holding a list of words.
	 */
	WordGUI(JFrame appFrame, WordListModel model) {
		// Save a reference to the frame for the GUI.
		frame = appFrame;

		// Save a reference to the data model for the program.
		listModel = model;

		// Setup the default close operation for the JFrame.
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Create A list to hold words in and initialize it.
		words = new JList(listModel.getVector());

		// Create a scroll pane to the hold the list.
		JScrollPane scrollPane = new JScrollPane(words);

		// Create JButtons for the program.
		addButton = new JButton("Add");
		addButton.addActionListener(this);

		removeButton = new JButton("Remove");
		removeButton.addActionListener(this);

		// Create panel to hold the buttons.
		JPanel buttonPanel = new JPanel();

		// Add buttons to the button panel.
		buttonPanel.add(addButton);
		buttonPanel.add(removeButton);

		// Add button panel and list to the frame.
		frame.getContentPane().add(scrollPane, BorderLayout.CENTER);
		frame.getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		// Size the frame and make it show up.
		frame.setSize(300, 400);
		frame.setVisible(true);
	}

	/**
	 * Implements the action listener for the GUI. Handles the events of clicking
	 * on the Add or Remove buttons.
	 * <P>
	 * 
	 * @param e
	 *            An object representing the Action Event that occurred.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == addButton) {
			// If the user clicked on Add, then add a word.
			addWord();
		} else if (e.getSource() == removeButton) {
			// If the user clicked on Remove, then remove the selected word.
			removeWord();
		}
	}

	/**
	 * Removes the selected word from the list of words.
	 */
	public void removeWord() {
		// Determine which word is selected.
		int index = words.getSelectedIndex();

		/*
		 * If index < 0, then no word is selected, so do nothing. Otherwise
		 * remove the selected word. A good modification here would be to
		 * display a message box to the user when they click on the remove
		 * button without first selecting an item in the list.
		 */
		if (index > -1) {
			listModel.remove(index);
			words.setListData(listModel.getVector());
		}
	}

	/**
	 * Adds a new word to the list of words.
	 */

	public void addWord() {
		// Create a data model to hold the new word.
		Word dataModel = new Word();

		/*
		 * Create and display a dialog box that allows the user to enter the
		 * data associated with the word.
		 */
		WordDialog dialog = new WordDialog(frame, dataModel);
		dialog.setVisible(true);

		/*
		 * Determine how the user closed the dialog. If they closed it by
		 * affirming that they want the data saved, then add the word to the
		 * list of words.
		 */
		if (dialog.getCloseOption() == WordDialog.OK_OPTION) {
			listModel.add(dataModel);
			words.setListData(listModel.getVector());
		}
	}

	/**
	 * This is the main method of the program. It creates an initial lists of
	 * movies and creates a GUI that allows the list to be manipulated.
	 * <P>
	 * 
	 * @param args
	 *            Any command line arguments. This program does not use command
	 *            line arguments.
	 */
	public static void main(String args[]) {
		/* Data model for the program. */
		WordListModel words;
		/* Frame for the GUI of the program. */
		JFrame guiFrame;
		/* Movie GUI for the program. */
		WordGUI gui;

		// Initialize the list words.
		words = new WordListModel();

		/*
		 * 1. Instantiate three objects of type Movie and add them to the list
		 * movies.
		 */
		Word word1 = new Word("Word1", "english");
		Word word2 = new Word("Word2", "spanish");
		Word word3 = new Word("Word3", "cantonese");
		// xx ...

		/*
		 * The list is referenced to by the variable words. You can add to the
		 * list by invoking the add method if ArrayList.
		 */
		words.add(word1);
		words.add(word2);
		words.add(word3);
		// xx ...

		// Create a frame for the program.
		guiFrame = new JFrame("Dictionary System");

		// Create GUI.
		gui = new WordGUI(guiFrame, words);

	}

	/**
	 * Uses the data model to store the inventory out to a user specified
	 * data file.
	 */
	private void saveAction()
	{
		boolean success = false;
		do
		{
			/* Query the user for the file to use.
			 * Start the dialog in the current directory by passing
			 * JFileChooser a . as the path.
			 */
			JFileChooser chooser = new JFileChooser(".");
			int returnVal = chooser.showSaveDialog(frame);
			if(returnVal == JFileChooser.APPROVE_OPTION) 
			{
				try {
					listModel.saveToFile( chooser.getSelectedFile().getPath() );
					success = true;
				} 
				catch( IOException ioe )
				{
					/* If there was an error saving the file, let the user know
					 * and give them a chance to try again.
					 */
					JOptionPane.showMessageDialog(frame,
					"Could Not Open File!" );
				}
			}
			else
			{
				success = true;
			}
		} while( success == false ); /* Loop iterates until the file has been
		 * successfully written or the user CANCELS the operation. */
	}

	/**
	 * Uses the data model to load the inventory from a user specified
	 * data file.
	 */
	private void loadAction()
	{
		boolean success = false;
		do
		{
			/* Query the user for the file to use.
			 * Start the dialog in the current directory by passing
			 * JFileChooser a . as the path.
			 */
			JFileChooser chooser = new JFileChooser(".");
			int returnVal = chooser.showOpenDialog(frame);
			if(returnVal == JFileChooser.APPROVE_OPTION) 
			{
				try {
					// Load from the file.
					listModel.loadFromFile( chooser.getSelectedFile().getPath() );

					// Update the display.
					words.setListData( listModel.getVector() );
					success = true;
				} 
				catch( IOException ioe )
				{
					/* If there was an error opening the file, let the user know
					 * and give them a chance to try again.
					 */
					JOptionPane.showMessageDialog(frame,
					"Could Not Open File!" );
				}
				catch( ClassNotFoundException cnfe )
				{
					/* If there was an error reading the file, let the user know
					 * and give them a chance to try again.
					 */
					JOptionPane.showMessageDialog(frame,
					"Could Not Read Selected File!" );
				}
			}
			else
			{
				success = true;
			}
		} while( success == false ); /* Loop iterates until the file has been
		 * successfully written or the user CANCELS the operation. */
	}


}
